@@ -1,18 +1,16 @@
-// =================================================================
-//          #     #                 #     #
-//          ##    #   ####   #####  ##    #  ######   #####
-//          # #   #  #    #  #    # # #   #  #          #
-//          #  #  #  #    #  #    # #  #  #  #####      #
-//          #   # #  #    #  #####  #   # #  #          #
-//          #    ##  #    #  #   #  #    ##  #          #
-//          #     #   ####   #    # #     #  ######     #
+// ==========================================================================
+//     _   _ _ ____            ____          _____
+//    | | | (_)  _ \ ___ _ __ / ___|___  _ _|_   _| __ __ _  ___ ___ _ __
+//    | |_| | | |_) / _ \ '__| |   / _ \| '_ \| || '__/ _` |/ __/ _ \ '__|
+//    |  _  | |  __/  __/ |  | |__| (_) | | | | || | | (_| | (_|  __/ |
+//    |_| |_|_|_|   \___|_|   \____\___/|_| |_|_||_|  \__,_|\___\___|_|
 //
-//       ---   The NorNet Testbed for Multi-Homed Systems  ---
-//                       https://www.nntb.no
-// =================================================================
+//       ---  High-Performance Connectivity Tracer (HiPerConTracer)  ---
+//                 https://www.nntb.no/~dreibh/hipercontracer/
+// ==========================================================================
 //
 // High-Performance Connectivity Tracer (HiPerConTracer)
-// Copyright (C) 2015-2022 by Thomas Dreibholz
+// Copyright (C) 2015-2026 by Thomas Dreibholz
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,10 +32,21 @@
 
 #include "destinationinfo.h"
 #include "resultentry.h"
+#include "resultswriter.h"
 
 #include <functional>
 
 
+enum OutputFormatVersionType
+{
+   OFT_HiPerConTracer_Version1 = 1,
+   OFT_HiPerConTracer_Version2 = 2,
+
+   OFT_Min = OFT_HiPerConTracer_Version1,
+   OFT_Max = OFT_HiPerConTracer_Version2
+};
+
+
 class Service;
 
 typedef std::function<void(Service* service, const ResultEntry* resultEntry)> ResultCallbackType;
@@ -46,7 +55,10 @@ typedef std::function<void(Service* service, const ResultEntry* resultEntry)> Re
 class Service
 {
    public:
-   Service();
+   Service(ResultsWriter*                resultsWriter,
+           const char*                   outputFormatName,
+           const OutputFormatVersionType outputFormatVersion,
+           const unsigned int            iterations);
    virtual ~Service();
 
    virtual const boost::asio::ip::address& getSource() = 0;
@@ -54,13 +66,18 @@ class Service
    virtual void setResultCallback(const ResultCallbackType& resultCallback);
 
    virtual const std::string& getName() const = 0;
+   virtual bool prepare(const bool privileged);
    virtual bool start() = 0;
    virtual void requestStop() = 0;
    virtual bool joinable() = 0;
    virtual void join() = 0;
 
    protected:
-   ResultCallbackType ResultCallback;
+   ResultCallbackType            ResultCallback;
+   ResultsWriter*                ResultsOutput;
+   const std::string             OutputFormatName;
+   const OutputFormatVersionType OutputFormatVersion;
+   const unsigned int            Iterations;
 };
 
 #endif
